'From Squeak 2.4b of April 23, 1999 on 28 April 1999 at 3:39:36 pm'!
"Change Set:		findWindow-sw
Date:			28 April 1999
Author:			Scott Wallace

Makes shift-click on the world background in morphic bring up the find-window menu, as it long has done in mvc.

In the morphic find-window menu, selections in each category (first expanded windows, then collapsed ones) are now presented in (case-insensitive) alphabetical order.

Also, in the morphic find-window menu, items representing windows that have unsubmitted changes now show in red."!

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 4/19/1999 12:37'!
findWindow
	"Present a menu of window titles, and activate the one that gets chosen.
	Collapsed windows appear below line, expand if chosen."
	| menu expanded collapsed |
	menu _ MenuMorph new.
	expanded _ SystemWindow windowsIn: self world satisfying: [:w | w isCollapsed not].
	collapsed _ SystemWindow windowsIn: self world satisfying: [:w | w isCollapsed].
	(expanded isEmpty & collapsed isEmpty) ifTrue: [^ self beep].
	(expanded asSortedCollection: [:w1 :w2 | w1 label caseInsensitiveLessOrEqual: w2 label]) do:
		[:w | menu add: w label target: w action: #activate.
			w model canDiscardEdits ifFalse: [menu lastItem color: Color red]].
	expanded isEmpty | collapsed isEmpty ifFalse: [menu addLine].
	(collapsed asSortedCollection: [:w1 :w2 | w1 label caseInsensitiveLessOrEqual: w2 label]) do: 
		[:w | menu add: w label target: w action: #collapseOrExpand.
		w model canDiscardEdits ifFalse: [menu lastItem color: Color red]].
	menu addTitle: 'find window'.
	
	self invokeMenu: menu event: lastEvent! !

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 4/19/1999 12:02'!
invokeMetaMenu: evt
	"Invoke the meta menu. If the hand is over the background, the world menu is presented. If it is over a morph, a menu of operations for that morph is presented. Each menu entry contains a string to be presented in the menu and a selector. If the selector takes an argument, the mouse-down event that invoked the menu is passed as an argument. This lets the command know which hand invoked it in  order to do things like attaching the result of the command to that hand.

If the hand is over the background and the shift key is pressed, the find-window menu is immediately put up"

	| menu |
	Preferences noviceMode ifTrue: [^ self].

	"if carrying morphs, just drop them"
	self hasSubmorphs ifTrue: [^ self dropMorphsEvent: evt].

	targetOffset _ menuTargetOffset _ self position.
	argument _ self argumentOrNil.
	argument == nil
		ifTrue:
			[Utilities bringFlapsToFront.
			evt shiftPressed
				ifFalse:
					["put put screen menu"
					menu _ self buildWorldMenu.
					menu addTitle: 'World']
				ifTrue:
					[^ self findWindow]]
		ifFalse:
			[menu _ self buildMorphMenuFor: argument.
			menu addTitle: argument class name].
	self invokeMenu: menu event: evt.
! !


!MenuMorph methodsFor: 'accessing' stamp: 'sw 4/19/1999 12:22'!
lastItem
	^ submorphs last! !


!String methodsFor: 'comparing' stamp: 'sw 4/19/1999 12:11'!
caseInsensitiveLessOrEqual: aString 
	"Answer whether the receiver sorts before or equal to aString.
	The collation order is case insensitive."

	^ (self compare: self with: aString collated: CaseInsensitiveOrder) <= 2! !
