'From Squeak 2.4c of May 10, 1999 on 4 June 1999 at 9:42:27 am'!
"Change Set:		ScreenMorphTweaks
Date:			3 June 1999
Author:			Dan Ingalls

Adds the ability of ScreeningMorphs to show the non-passing part of their screenMorph.
They respond to layoutChanged, so that they properly reflect, eg, border width changes.
When in screened mode, they do not respond outside the screenMorph bounds.
Drag and Drop is enabled when they have less than 2 submorphs.
A default passingColor is chosen from the senter of the screenMorph when it is added.
Use of the brown handle now works on either submorph.
Finally, the screenForm logic has been tweaked for proper offsets.
"!

Morph subclass: #ScreeningMorph
	instanceVariableNames: 'screenForm displayMode passingColor passElseBlock '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Demo'!

!BorderedMorph methodsFor: 'accessing' stamp: 'di 6/4/1999 09:42'!
borderWidth: anInteger
	borderColor ifNil: [borderColor _ Color black].
	borderWidth _ anInteger max: 0.
	self changed! !


!ScreeningMorph methodsFor: 'initialization' stamp: 'di 6/4/1999 08:25'!
addMorph: aMorph

	| f |
	super addMorph: aMorph.
	submorphs size <= 2 ifTrue:
		[self bounds: submorphs last bounds].
	submorphs size = 2 ifTrue:
		["The screenMorph has just been added.
		Choose as the passingColor the center color of that morph"
		f _ self screenMorph imageForm.
		passingColor _ f colorAt: f boundingBox center.
		passElseBlock _ true]! !

!ScreeningMorph methodsFor: 'initialization' stamp: 'di 6/4/1999 07:17'!
initialize
	super initialize.
	passingColor _ Color black.
	passElseBlock _ true.
	displayMode _ #showScreened.
	self openDragAndDrop! !

!ScreeningMorph methodsFor: 'initialization' stamp: 'di 6/4/1999 08:27'!
privateRemoveMorph: aMorph

	super privateRemoveMorph: aMorph.
	submorphs size = 1 ifTrue:
		[self bounds: submorphs first bounds]! !

!ScreeningMorph methodsFor: 'menu' stamp: 'di 6/3/1999 16:41'!
chooseBlockingColor
	passingColor _ Color fromUser.
	passElseBlock _ false.
	self layoutChanged! !

!ScreeningMorph methodsFor: 'menu' stamp: 'di 6/3/1999 16:41'!
choosePassingColor
	passingColor _ Color fromUser.
	passElseBlock _ true.
	self layoutChanged! !

!ScreeningMorph methodsFor: 'drawing' stamp: 'di 6/4/1999 07:31'!
fullDrawOn: aCanvas
	| mergeForm |
	(aCanvas isVisible: self fullBounds) ifFalse: [^ self].
	submorphs size = 0 ifTrue: [^ super fullDrawOn: aCanvas].
	(submorphs size = 1 or: [displayMode == #showScreenOnly]) ifTrue:
		[^ self screenMorph fullDrawOn: aCanvas].
	displayMode == #showSourceOnly ifTrue:
		[^ self sourceMorph fullDrawOn: aCanvas].
	displayMode == #showScreenOverSource ifTrue:
		[self sourceMorph fullDrawOn: aCanvas.
		^ self screenMorph fullDrawOn: aCanvas].
	displayMode == #showScreened ifTrue:
		[self screenMorph fullDrawOn: aCanvas.
		mergeForm _ self sourceMorph imageFormForRectangle: self screenMorph bounds.
		(BitBlt toForm: mergeForm) copyForm: self screenForm to: 0@0 rule: Form and
			colorMap: (Bitmap with: 0 with: 16rFFFFFFFF).
		aCanvas image: mergeForm at: self screenMorph position]! !

!ScreeningMorph methodsFor: 'geometry' stamp: 'di 6/4/1999 09:03'!
layoutChanged

	screenForm _ nil.
	submorphs size >= 2
		ifTrue: [self closeDragAndDrop]
		ifFalse: [self openDragAndDrop].
	submorphs size = 2 ifTrue:
		[bounds _ ((self sourceMorph bounds merge: self screenMorph bounds) expandBy: 4)].
	^ super layoutChanged! !

!ScreeningMorph methodsFor: 'geometry' stamp: 'di 6/4/1999 09:28'!
morphsAt: aPoint addTo: mList
	"Impose screenMorph clipping when screening is in effect"

	(submorphs size = 2
		and: [displayMode == #showScreened
		and: [(self containsPoint: aPoint) not]]) ifTrue: [^ mList].
	^ super morphsAt: aPoint addTo: mList
! !

!ScreeningMorph methodsFor: 'geometry' stamp: 'di 6/4/1999 09:28'!
unlockedMorphsAt: aPoint addTo: mList
	"Impose screenMorph clipping when screening is in effect"

	(submorphs size = 2
		and: [displayMode == #showScreened
		and: [(self containsPoint: aPoint) not]]) ifTrue: [^ mList].
	^ super unlockedMorphsAt: aPoint addTo: mList
! !

!ScreeningMorph methodsFor: 'private' stamp: 'di 6/3/1999 19:31'!
screenForm

	| screenImage colorMap pickValue elseValue |
	screenForm ifNotNil: [^ screenForm].
	passElseBlock ifNil: [passElseBlock _ true].
	passingColor ifNil: [passingColor _ Color black].
	passElseBlock
		ifTrue: [pickValue _ 16rFFFFFFFF.  elseValue _ 0]
		ifFalse: [pickValue _ 0.  elseValue _ 16rFFFFFFFF].
	screenImage _ self screenMorph imageFormForRectangle: self screenMorph bounds.
	colorMap _ screenImage newColorMap atAllPut: elseValue.
	colorMap at: (passingColor indexInMap: colorMap) put: pickValue.
	screenForm _ Form extent: screenImage extent.
	screenForm copyBits: screenForm boundingBox
			from: screenImage at: 0@0 colorMap: colorMap.
	^ screenForm! !


ScreeningMorph removeSelector: #mapColor:to:othersTo:!

ScreeningMorph allInstancesDo: [:sm | sm layoutChanged]!
