'From Squeak 2.1 of June 30, 1998 on 22 August 1998 at 11:51:27 am'!
"Change Set:		jhmLowSpaceBug
Date:			20 August 1998
Author:			John Maloney

This fixes a bug in the handling of low-space condtions that crept in
with Morphic 2.0. The problem only occurs in a MVC projects. The
problem is that the Debugger was not being told to release itself anymore,
and thus it was not restarting the low-space watcher process. So, the
first low-space condition would be caught, but the next would be fatal.

This fix adds a mechanism to inform models when their window
is being closed (that's how release was being used before). This is
somewhat in tension with the MVC goal of supporting multiple views,
but as long as it is used sparingly (e.g., for the debugger), it should
be okay."
!


!Object methodsFor: 'updating' stamp: 'jm 8/20/1998 18:26'!
windowIsClosing
	"This message is used to inform a models that its window is closing. Most models do nothing, but some, such as the Debugger, must do some cleanup. Note that this mechanism must be used with care by models that support multiple views, since one view may be closed while others left open."
! !


!Celeste methodsFor: 'open-close' stamp: 'jm 8/20/1998 18:39'!
close
	"Close the mail database."

	mailDB close; release.
	mailDB _ nil.
! !

!Celeste methodsFor: 'open-close' stamp: 'jm 8/20/1998 18:37'!
windowIsClosing
	"Close the mail database when my window is closed."

	self close.
! !

!Celeste methodsFor: 'categories pane' stamp: 'jm 8/20/1998 18:34'!
save
	"Snapshot the database to disk."

	mailDB saveDB.
! !


!Debugger methodsFor: 'initialize' stamp: 'jm 8/20/1998 18:31'!
release

	self windowIsClosing.
	super release.
! !

!Debugger methodsFor: 'initialize' stamp: 'jm 8/20/1998 18:30'!
windowIsClosing
	"My window is being closed; clean up. Restart the low space watcher."

	interruptedProcess == nil ifTrue: [^ self].
	interruptedProcess terminate.
	interruptedProcess _ nil.
	interruptedController _ nil.
	contextStack _ nil.
	contextStackTop _ nil.
	receiverInspector _ nil.
	contextVariablesInspector _ nil.
	Smalltalk installLowSpaceWatcher.  "restart low space handler"
! !


!StandardSystemView methodsFor: 'initialize-release' stamp: 'jm 8/20/1998 18:29'!
release

	model windowIsClosing.
	self isCollapsed ifTrue: [savedSubViews do: [:v | v release]].
	super release.
! !


!SystemWindow methodsFor: 'open/close' stamp: 'jm 8/20/1998 18:32'!
delete
	| thisWorld |
	model okToChange ifFalse: [^self].
	thisWorld _ self world.
	model windowIsClosing; release.
	model _ nil.
	super delete.
	SystemWindow noteTopWindowIn: thisWorld.
! !


