'From Squeak2.9alpha of 26 July 2000 [latest update: #2574] on 8 September 2000 at 8:53:56 pm'!
"Change Set:		worldMorphMenu-sw
Date:			8 September 2000
Author:			Scott Wallace

The world's halo now gives access to the regular halo menu for the World thought of as just another morph (up until now, it had always put up the desktop menu); there's an item in that world-halo menu, which will take you to the desktop menu.
 (At Nathanael Schaerli's suggestion)"!


!Morph methodsFor: 'menus' stamp: 'sw 9/1/2000 06:59'!
addCustomMenuItems: aCustomMenu hand: aHandMorph
	"Add morph-specific items to the given menu which was invoked by the given hand."

	self isWorldMorph
		ifFalse:
			[aCustomMenu add: 'add drop-shadow' action: #addDropShadow.
			(self isKindOf: SystemWindow)
				ifFalse: [aCustomMenu add: 'put in a window' action: #embedInWindow].
			aCustomMenu addUpdating: #stickinessString target: self action: #toggleStickiness.
			aCustomMenu add: 'adhere to edge...' action: #adhereToEdge]
		ifTrue:
			[aCustomMenu add: 'desktop menu...' target: aHandMorph action: #putUpDesktopMenu.
			aCustomMenu addLine].

	Preferences noviceMode ifFalse:
		[self addDebuggingItemsTo: aCustomMenu hand: aHandMorph].
	
! !


!BorderedMorph methodsFor: 'menu' stamp: 'sw 9/6/2000 05:14'!
addCustomMenuItems: aMenu hand: aHandMorph
	super addCustomMenuItems: aMenu hand: aHandMorph.
	self isWorldMorph ifFalse:
		[aMenu addList: 
			#(('border color...' changeBorderColor:)
			('border width...' changeBorderWidth:)).
		self couldHaveRoundedCorners ifTrue:
			[aMenu addUpdating: #roundedCornersString target: self action: #toggleCornerRounding].

		self doesBevels ifTrue:
			[borderColor == #raised ifFalse: [aMenu add: 'raised bevel' action: #borderRaised].
			borderColor == #inset ifFalse: [aMenu add: 'inset bevel' action: #borderInset]]]
! !


!HandMorph methodsFor: 'world menu' stamp: 'sw 9/6/2000 05:20'!
putUpDesktopMenu
	"Put up the desktop menu"

	^ (self buildWorldMenu addTitle: Preferences desktopMenuTitle) popUpForHand: self

	"self currentHand putUpDesktopMenu"
! !

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 9/1/2000 06:29'!
buildMorphHandleMenuFor: argMorph
	"Build the morph menu for the given morph's halo's menu handle. This menu has two sections. The first section contains commands that are interpreted by the hand; the second contains commands provided by the target morph. This method allows the morph to decide which items should be included in the hand's section of the menu."

	| menu |

	argument _ argMorph.
	menu _ MenuMorph new defaultTarget: self.
	menu addStayUpItem.
	argMorph addAddHandMenuItemsForHalo: menu hand: self.
	menu defaultTarget: argMorph.
	argMorph addCustomHaloMenuItems: menu hand: self.
	menu addLine.

	argMorph player ifNotNil: [argMorph player addPlayerMenuItemsTo: menu hand: self].
	menu defaultTarget: self.

	^ menu
! !

!HandMorph methodsFor: 'world menu commands' stamp: 'sw 9/6/2000 05:19'!
invokeMetaMenu: evt
	"Invoke the meta menu. If the hand is over the background, the world menu is presented. If it is over a morph, a menu of operations for that morph is presented. Each menu entry contains a string to be presented in the menu and a selector. If the selector takes an argument, the mouse-down event that invoked the menu is passed as an argument. This lets the command know which hand invoked it in order to do things like attaching the result of the command to that hand.

If the hand is over the background and the shift key is pressed, the find-window menu is immediately put up.

If the hand is over the background but the yellow button was pressed, an alternate menu, which individual users are encouraged to personalize, is put up -- see HandMorph.yellowButtonClickOnDesktopWithEvent:"

	| menu |
	Preferences metaMenuDisabled ifTrue: [^ self].

	"if carrying morphs, just drop them"
	self hasSubmorphs ifTrue: [^ self dropMorphsEvent: evt].

	self flag: #bob.		"do we need this?"
	targetOffset _ menuTargetOffset _ self worldUnderCursor globalPointToLocal: self position.
	argument _ self argumentOrNil.
	argument == nil
		ifTrue:
			[Utilities bringFlapsToFront.
			evt yellowButtonPressed
				ifTrue: [^ self yellowButtonClickOnDesktopWithEvent: evt].

			evt shiftPressed
				ifFalse:
					["put put screen menu"
					menu _ self buildWorldMenu.
					menu addTitle: Preferences desktopMenuTitle]
				ifTrue:
					[^ self findWindow]]
		ifFalse:
			[menu _ self buildMorphMenuFor: argument.
			menu addTitle: argument class name].
	menu popUpEvent: evt.
! !


!PasteUpMorph methodsFor: 'menu & halo' stamp: 'sw 9/1/2000 06:47'!
addAddHandMenuItemsForHalo: aMenu hand: aHandMorph
	"Add halo menu items to be handled by the invoking hand. The halo menu is invoked by clicking on the menu-handle of the receiver's halo.  Copied down from morph and hand-edited for the world"

	self isWorldMorph ifFalse: [^ super addAddHandMenuItemsForHalo: aMenu hand: aHandMorph].

	aMenu addLine.
	aMenu add: 'make a selection' action: #makeSelection.
	aMenu add: 'copy Postscript' action: #clipPostscript.
	aMenu add: 'print PS to file...' target: self selector: #printPSToFile.
	aMenu addLine.
	self addFillStyleMenuItems: aMenu hand: aHandMorph.
	aMenu
		defaultTarget: self;
		add: 'add mouse up action' action: #addMouseUpAction;
		add: 'remove mouse up action' action: #removeMouseUpAction;
		defaultTarget: aHandMorph.
! !


!Preferences class methodsFor: 'hard-coded prefs' stamp: 'sw 9/6/2000 05:26'!
desktopMenuTitle
	"Answer the title to be used for the 'meta menu'.  For now, you can hard-code this, later someone should make this be a parameter the user can easily change.  sw 9/6/2000"

	^ 'World'    "This is what it has always been"

	"^ 'Desktop'
	^ 'Squeak'
	^ 'Mike''s Control Panel'"! !

