'From Squeak 2.2beta of Sept 11, 1998 on 19 September 1998 at 10:47:53 pm'!
"Change Set:		prefsCleanup-sw
Date:			19 September 1998
Author:			Scott Wallace

Consolidates various mechanisms into a single one for maintaining and retrieving boolean-valued preferences."!


!CautiousModel methodsFor: 'all' stamp: 'sw 9/15/1998 16:45'!
okToChange
	Preferences cautionBeforeClosing ifFalse: [^ true].
	Sensor leftShiftDown ifTrue: [^ true].

	self beep.
	^ self confirm: 'Warning!!
If you answer "yes" here, this
window will disappear and
its contents will be lost!!
Do you really want to do that?'

"CautiousModel new okToChange"! !


!ChangeSet methodsFor: 'fileIn/Out' stamp: 'sw 9/15/1998 16:52'!
fileOut
	"File out the receiver, to a file whose name is a function of the change-set name and of the date and the time."
	"wod 5/12/1998: use #asFileName rather than #truncateTo:"

	| file slips |
	Cursor write showWhile:
		[file _ FileStream newFileNamed:
			(self name, FileDirectory dot, Utilities dateTimeSuffix, FileDirectory dot, 'cs') asFileName.
		file header; timeStamp.
		self fileOutPreambleOn: file.
		self fileOutOn: file.
		self fileOutPostscriptOn: file.
		file trailer; close].

	Preferences suppressCheckForSlips ifTrue: [^ self].

	slips _ self checkForSlips.
	(slips size > 0 and: [self confirm: 'Methods in this fileOut have halts
or references to the Transcript
or other ''slips'' in them.
Would you like to browse them?'])
		ifTrue: [Smalltalk browseMessageList: slips
							name: 'Possible slips in ', name]! !


!EToySystem class methodsFor: 'housekeeping' stamp: 'sw 9/15/1998 16:22'!
prepareRelease
	"To prepare a release of the EToy system, first make a copy of your image. Start that image and:
		1. close all windows and projects
		2. clean any garbage out of the following:
			Smalltalk at: #ImageImports.
			(Smalltalk keys select: [:k | ((Smalltalk at: k) isKindOf: Behavior) not])
				asSortedCollection asArray.
			SampledSound soundLibrary.
		3. run this method
		4. save the image"
	"EToySystem prepareRelease"

	HandMorph initialize.  "free cached ColorChart"
	PaintBoxMorph releaseTemporaryForms.
	PaintBoxMorph prototype stampHolder clear.  "clear stamps"
	PaintBoxMorph prototype delete.  "break link to world, if any"
	Smalltalk removeKey: #AA ifAbsent: [].
	Smalltalk removeKey: #BB ifAbsent: [].
	Smalltalk removeKey: #CC ifAbsent: [].
	Smalltalk removeKey: #DD ifAbsent: [].
	Smalltalk removeKey: #Temp ifAbsent: [].

 	Preferences setPreference: #quitWhenExitingImagineeringStudio toValue: true.
	Preferences setPreference: #showDevelopersEToys toValue: false.
	Preferences setPreference: #startImagineeringStudio toValue: true.
	Preferences setPreference: #updateFromServer toValue: false.
	Preferences setPreference: #warnIfNoChangesFile toValue: false.
	Preferences setPreference: #warnIfNoSourcesFile toValue: false.

	Smalltalk minorShrink.
	SystemOrganization removeSystemCategory: 'Morphic-Windows'.
	SystemOrganization removeSystemCategory: 'Graphics-Files'. "No license for GIF reader"
	ScriptingSystem reclaimSpace.
	Smalltalk cleanOutUndeclared.
	Smalltalk reclaimDependents.
	Smalltalk forgetDoIts.
	Smalltalk removeEmptyMessageCategories.
	Socket deadServer: ''.	"Don't reveal any specific server name"
	Symbol rehash.

	Preferences desktopColor: (Color r: 0.2 g: 0.2 b: 0.2).
	ScheduledControllers updateGray; restore.
! !

!EToySystem class methodsFor: 'external release' stamp: 'sw 9/15/1998 16:58'!
methodsToStripForExternalRelease
	"Answer a list of triplets #(className, class/instance, methodName) of methods to be stripped in an external release."
	^ #(

		(EToySystem			class		serverUrls)
		(EToySystem			class		eToyDoItStrings)
		(EToySystem			class		loadPurpleWalt)	
		(EToySystem			class		readCollagePic)	
		(EToySystem			class		prepareRelease)	
		(EToySystem			class		previewEToysOn:)
		)! !


!Preferences class reorganize!
('initialization' chooseInitialSettings initialize)
('parameters' defaultAuthorName desktopColor desktopColor: fontForScriptorButtons scrollBarColor scrollBarWidth)
('preferences dictionary' deletePreference: doesNotUnderstand: openPreferencesInspector setPreference:toValue: valueOfFlag:)
!


!Preferences class methodsFor: 'initialization' stamp: 'sw 9/15/1998 16:44'!
chooseInitialSettings
	"Restore the default choices for Preferences."
	"Preferences chooseInitialSettings"

	self setPreference: #cautionBeforeClosing toValue: true.
	self setPreference: #cmdDotEnabled toValue: true.
	self setPreference: #confirmFirstUseOfStyle toValue: true.
	self setPreference: #disableSounds toValue: false.
	self setPreference: #editPlayerScriptsInPlace toValue: false.
	self setPreference: #ignoreStyleIfOnlyBold toValue: true.
	self setPreference: #noviceMode toValue: false.
	self setPreference: #reverseWindowStagger toValue: true.
	self setPreference: #showDebugHaloHandle toValue: true.
	self setPreference: #showTimeStampsInMenuTitles toValue: true.
	self setPreference: #suppressCheckForSlips toValue: false.
	self setPreference: #thoroughSenders toValue: true.
	self setPreference: #uniformWindowColors toValue: false.
	self setPreference: #warnIfNoChangesFile toValue: true.
	self setPreference: #warnIfNoSourcesFile toValue: true.

"	self setPreference: #xxx toValue: false.
	self setPreference: #xxx toValue: false."
! !

!Preferences class methodsFor: 'initialization' stamp: 'sw 9/19/1998 22:47'!
initialize
	"Preferences initialize"
	"Sometimes placed in a change-set even though unchanged, to trigger reinitialization upon update."

	FlagDictionary _ Dictionary new.
	self chooseInitialSettings.
! !


!Project methodsFor: 'active process' stamp: 'sw 9/15/1998 16:42'!
maybeForkInterrupt
	Preferences cmdDotEnabled ifTrue:
		[Smalltalk isMorphic
			ifTrue: [[self interruptName: 'User Interrupt'] fork]
			ifFalse: [[ScheduledControllers interruptName: 'User Interrupt'] fork]]! !


EToySystem class removeSelector: #loadRedWalt!
EToySystem class removeSelector: #openEToyControls!
EToySystem class removeSelector: #releaseStuff!
EToySystem class removeSelector: #formDictExpressions!
Preferences class removeSelector: #noviceMode!
Preferences class removeSelector: #userPlacement!
Preferences class removeSelector: #editPlayerScriptsInPlace!
Preferences class removeSelector: #showDebugHaloHandle!
Preferences class removeSelector: #showTimeStampsInMenuTitles!
Preferences initialize!
Preferences class removeSelector: #suppressCheckForSlips!
Preferences class removeSelector: #confirmFirstUseOfStyle!
Preferences class removeSelector: #ignoreStyleIfOnlyBold!
ScreenController removeSelector: #openEToyPanel!
Smalltalk removeClassNamed: #EToyParameters!
