'From Squeak2.9alpha of 5 August 2000 [latest update: #3138] on 10 December 2000 at 1:33:41 pm'!
"Change Set:		threeWindowCommands-sw
Date:			10 December 2000
Author:			Scott Wallace

Adds three items,  'find a transcript' , 'close window', and  'send window to back', to the 'windows & flaps' menu, and provides keyboard accelerators (cmd-t, cmd-w, and cmd-\, respectively) for them.  

'Find a transcript' will bring an open Transcript to the front, even if it has to create a new one or expand a collapsed one.  Simply type cmd-t.

The close-window and send-window-to-back items apply to the active window.

Send-top-window-to-back is especially useful when used with a windows-and-flaps menu that has been 'kept up', and also when issued by repeated hitting of cmd-\, in which case it lets you cycle through all your windows without ever having to touch the mouse.  

Cmd-\ and cmd-w together provides simple, mouseless desktop housekeeping.

Since new cmd-key mappings are involved, the contents of the 'command-key-help' window are updated, and some unreleated tweaks are made there as well."!


!PasteUpMorph methodsFor: 'world menu' stamp: 'sw 12/6/2000 20:28'!
findATranscript: evt
	"Locate a transcript, open it, and bring it to the front.  Create one if necessary"

	| aWindow |
	submorphs do:
		[:aMorph | (((aWindow _ aMorph renderedMorph) isKindOf: SystemWindow) and:
			[aWindow model == Transcript])
				ifTrue:
					[aWindow isCollapsed ifTrue: [aWindow expand].
					aWindow activateAndForceLabelToShow.
					^ self]].
	"None found, so create one"
	(Transcript openAsMorphLabel: 'Transcript') openInWorld: evt hand world! !

!PasteUpMorph methodsFor: 'world menu' stamp: 'sw 12/6/2000 20:22'!
keystrokeInWorld: evt
	"A keystroke was hit when no keyboard focus was in set, so it is sent here to the world instead"

	|  aChar isCmd |
	aChar _ evt keyCharacter.
	isCmd _ evt commandKeyPressed and: [Preferences cmdKeysInText].
	isCmd ifTrue:
		[(aChar == $b) ifTrue: [^ Browser openBrowser].          "for example"
		(aChar == $z) ifTrue: [^ self commandHistory undoOrRedoCommand].
		(aChar == $w) ifTrue: [^ SystemWindow closeTopWindow].
		(aChar == $\) ifTrue: [^ SystemWindow sendTopWindowToBack].
		(aChar == $t) ifTrue: [^ self findATranscript: evt].
		(aChar == $W) ifTrue: [^ self invokeWorldMenu: evt]]
			"This last item is a weirdo feature requested by the Open School in Fall of 2000 as a keyhole to the world menu in systems that normally do not offer a world menu"! !


!SystemWindow class methodsFor: 'top window' stamp: 'sw 12/6/2000 20:13'!
closeTopWindow
	"Try to close the top window.  It may of course decline"

	TopWindow ifNotNil:
		[TopWindow delete]! !

!SystemWindow class methodsFor: 'top window' stamp: 'sw 12/6/2000 19:43'!
sendTopWindowToBack
	"Send the top window of the world to the back, activating the one just beneath it"

	TopWindow ifNotNil:
		[TopWindow sendToBack]! !


!TheWorldMenu methodsFor: 'windows & flaps menu' stamp: 'sw 12/6/2000 19:45'!
windowsMenu
        "Build the windows menu for the world."

        ^ self fillIn: (self menu: 'windows & flaps...') from: {  
                { 'find window' . { #myWorld . #findWindow: }. 'Presents a list of all windows; if you choose one from the list, it becomes the active window.'}.

                { 'find changed browsers...' . { #myWorld . #findDirtyBrowsers: }. 'Presents a list of browsers that have unsubmitted changes; if you choose one from the list, it becomes the active window.'}.

                { 'find changed windows...' . { #myWorld . #findDirtyWindows: }. 'Presents a list of all windows that have unsubmitted changes; if you choose one from the list, it becomes the active window.'}.


                { 'find a transcript' . { #myWorld . #findATranscript: }. 'Brings an open Transcript to the front, creating one if necessary, and makes it the active window'}.
			 nil.

                { #staggerPolicyString . { self . #toggleWindowPolicy }. 'stagger: new windows positioned so you can see a portion of each one.
                tile: new windows positioned so that they do not overlap others, if possible.'}.

                nil.
                { 'collapse all windows' . { #myWorld . #collapseAll }. 'Reduce all open windows to collapsed forms that only show titles.'}.
                { 'expand all windows' . { #myWorld . #expandAll }. 'Expand all collapsed windows back to their expanded forms.'}.
                { 'close top window (w)' . { SystemWindow . #closeTopWindow }. 'Close the topmost window if possible.'}.
                { 'send top window to back (\)' . { SystemWindow . #sendTopWindowToBack  }. 'Make the topmost window become the backmost one, and activate the window just beneath it.'}.

                nil.
                { 'delete unchanged windows' . { #myWorld . #closeUnchangedWindows }. 'Deletes all windows that do not have unsaved text edits.'}.
                { 'delete non-windows' . { #myWorld . #deleteNonWindows }. 'Deletes all non-window morphs lying on the world.'}.
                { 'delete both of the above' . { self .#cleanUpWorld }. 'deletes all unchanged windows and also all non-window morphs lying on the world, other than flaps.'}.

                nil.
            "    { #suppressFlapsString . { self . #toggleFlapSuppressionInProject }. 'Governs whether flaps should be shown in this project'}."
                { #useGlobalFlapsString . { self. #toggleWhetherToUseGlobalFlaps }. 'Governs whether a universal set of "global" flaps should be sharable by all morphic projects.'}.
			{ #whichGlobalFlapsString . { Utilities. #offerGlobalFlapsMenu }. 'Put up a menu that allows you to choose which global flaps to show in this project'. #globalFlapsEnabled}.

                { #newGlobalFlapString  . { Utilities . #addGlobalFlap }. 'Create a new flap that will be shared by all morphic projects'.  #globalFlapsEnabled}.
                nil.

                { 'new project flap...'  . { Utilities . #addLocalFlap }. 'Create a new flap to be used only in this project.'}.
                { 'add stack-tools flap'  . { Utilities . #addStackToolsFlap }. 'Add a flap in this project that offers tools for creating stacks and cards.'}.
				nil.

                { 'about flaps...' . { Utilities . #explainFlaps }. 'Gives a window full of details about how to use flaps.'}.

        }! !


!Utilities class methodsFor: 'support windows' stamp: 'sw 12/6/2000 20:30'!
commandKeyMappings
	^ self class firstCommentAt: #commandKeyMappings

"Lower-case command keys
(use with Cmd key on Mac and Alt key on other platforms)

a	Select all
b	Browse it (selection is a class name or cursor is over a class-list or message-list)
c	Copy selection
d	Do it (selection is a valid expression)
e	Exchange selection with prior selection
f	Find
g	Find again
h	Set selection as search string for find again
i	Inspect it (selection is a valid expression, or selection is over an inspect-ilst)
j	Again once (do the last text-related operation again)
k	Set font
l	Cancel
m	Implementors of it (selection is a message selector or cursor is over a class-list or message-list)
n	Senders of it (selection is a message selector or cursor is over a class-list or message-list)
o	Spawn current method
p	Print it (selection is a valid expression)
q	Query symbol (toggle all possible completion for a given prefix)
r	Recognizer
s	Save (i.e. accept)
t	Finds a Transcript (when cursor is over the desktop)
u	Toggle alignment
v	Paste
w	Delete preceding word (over text);  Close-window (over morphic desktop)
x	Cut selection
y	Swap characters
z	Undo

Note: for Do it, Senders of it, etc., a null selection will be expanded to a word or to the current line in an attempt to do what you want.  Also note that Senders/Implementors of it will find the outermost keyword selector in a large selection, as when you have selected a bracketed expression or an entire line.  Finally note that the same cmd-m and cmd-n (and cmd-v for versions) work in the message pane of most browsers.

Upper-case command keys
(use with Shift-Cmd, or Ctrl on Mac
 or Shift-Alt on other platforms; sometimes Ctrl works too)

A	Advance argument
B	Browse it in this same browser (in System browsers only)
C	Compare argument to clipboard
D	Duplicate
E	Method strings containing it
F	Insert 'ifFalse:'
I	Inspect via Object Explorer
J	Again many (apply the previous text command repeatedly until the end of the text)
K	Set style
L	Outdent (move selection one tab-stop left)
N	References to it (selection is a class name, or cursor is over a class-list or message-list)
O	Open single-message browser (in message lists)
P	Make project link
R	Indent (move selection one tab-stap right)
S	Search
T	Insert 'ifTrue:'
U	Convert linefeeds to carriage returns in selection
V	Paste author's initials
W	Selectors containing it (in text); show-world-menu (when issued with cursor over desktop)
X	Force selection to lowercase
Y	Force selection to uppercase
Z	Capitalize all words in selection

Other special keys

Backspace	Backward delete character
Del			Forward delete character
Shift-Bcksp	Backward delete word
Shift-Del	Forward delete word
Esc			Select current type-in

\			Send top window to back

Cursor keys
left, right,
up, or
down		Move cursor left, right, up or down
Ctrl+Left	Move cursor left one word
Ctrl+Right	Move cursor right one word
Home		Move cursor to begin of line or begin of text
End			Move cursor to end of line or end of text
PgUp, or
Ctrl+Up		Move cursor up one page
PgDown, or
Ctrl+Down	Move cursor down one page

Note all these keys can be used together with Shift to define or enlarge the selection. You cannot however shrink that selection again, which is, compared to other systems, still a limitation aka bug.

Other Cmd-key combinations (does not work on all platforms)

Return		Insert return followed by as many tabs as the previous line
			(with a further adjustment for additional brackets in that line)
Space		Select the current word as with double clicking

Enclose the selection in a kind of bracket.  Each is a toggle.
(does not work on all platforms)
Ctrl-(	Enclose within ( and ), or remove enclosing ( and )
Ctrl-[	Enclose within [ and ], or remove enclosing [ and ]
Crtl-{	Enclose within { and }, or remove enclosing { and }
Ctrl-<	Enclose within < and >, or remove enclosing < and >
Ctrl-'	Enclose within ' and ', or remove enclosing ' and '
Ctrl-""	Enclose within "" and "", or remove enclosing "" and ""

Note also that you can double-click just inside any of the above delimiters (or at the beginning or end of a line) to select the text enclosed.

Text Emphasis...
(does not work on all platforms)
Cmd-1	10 point font
Cmd-2	12 point font
Cmd-3	18 point font
Cmd-4	24 point font
Cmd-5	36 point font
Cmd-6	color, action-on-click, link to class comment, link to method, url
		Brings up a menu.  To remove these properties, select
		more than the active part and then use command-0.
Cmd-7	bold
Cmd-8	italic
Cmd-9	narrow (same as negative kern)
Cmd-0	plain text (resets all emphasis)
Cmd--	underlined (toggles it)
Cmd-=	struck out (toggles it)

Shift-Cmd--	(aka _) negative kern (letters 1 pixel closer)
Shift-Cmd-+	positive kern (letters 1 pixel larger spread)
"! !

"Postscript:"
Utilities replaceMenuFlap.
!

