'From Squeak3.3alpha of 28 January 2002 [latest update: #4780] on 2 March 2002 at 2:36:24 pm'!
"Change Set:		padSignature
Date:			2 March 2002
Author:			Bob Arning

Since the project signing mechanism expects each part of the signature to be exactly 20 bytes, add padding for those (relatively rare) cases where only 19 bytes are generated"!


!LargePositiveInteger methodsFor: 'converting' stamp: 'RAA 3/2/2002 14:32'!
withAtLeastNDigits: desiredLength

	| new |

	self size >= desiredLength ifTrue: [^self].
	new _ self class new: desiredLength.
	new
		replaceFrom: 1 
		to: self size 
		with: self 
		startingAt: 1.
	^new! !


!SecurityManager methodsFor: 'security operations' stamp: 'RAA 3/2/2002 14:33'!
positionToSecureContentsOf: aStream
	| bytes trusted part1 part2 sig hash dsa okay pos |
	aStream binary.
	pos _ aStream position.
	bytes _ aStream next: 4.
	bytes = 'SPRJ' asByteArray ifFalse:[
		"was not signed"
		aStream position: pos.
		^false].
	part1 _ (aStream nextInto: (LargePositiveInteger basicNew: 20)) normalize.
	part2 _ (aStream nextInto: (LargePositiveInteger basicNew: 20)) normalize.
	sig _ Array with: part1 with: part2.
	hash _ SecureHashAlgorithm new hashStream: aStream.
	dsa _ DigitalSignatureAlgorithm new.
	trusted _ self trustedKeys.
	okay _ (trusted detect:[:key| dsa verifySignature: sig ofMessageHash: hash publicKey: key]
			ifNone:[nil]) notNil.
	aStream position: pos+44.
	^okay! !

!SecurityManager methodsFor: 'security operations' stamp: 'RAA 3/2/2002 14:32'!
signFile: fileName directory: fileDirectory
	"Sign the given project in the directory"
	| bytes file dsa hash sig key |
	Preferences signProjectFiles ifFalse:[^self]. "signing turned off"
	key _ self signingKey.
	key ifNil:[^self].
	file _ FileStream readOnlyFileNamed: (fileDirectory fullNameFor: fileName).
	bytes _ file binary; contentsOfEntireFile.
	fileDirectory deleteFileNamed: fileName ifAbsent:[].
	dsa _ DigitalSignatureAlgorithm new.
	dsa initRandom: Time millisecondClockValue + Date today julianDayNumber.
	hash _ SecureHashAlgorithm new hashStream: (ReadStream on: bytes).
	sig _ dsa computeSignatureForMessageHash: hash privateKey: key.
	file _ FileStream newFileNamed: (fileDirectory fullNameFor: fileName).
	file binary.
	"store a header identifying the signed file first"
	file nextPutAll: 'SPRJ' asByteArray.
	"now the signature"
	file 
		nextPutAll: (sig first withAtLeastNDigits: 20); 
		nextPutAll: (sig last withAtLeastNDigits: 20).
	"now the contents"
	file nextPutAll: bytes.
	file close.! !

