'From Squeak 2.3 beta of Nov 25, 1998 on 23 December 1998 at 4:22:10 pm'!
"Change Set:		ThumbnailNav
Date:			23 December 1998
Author:			Dan Ingalls

Makes it possible to use BookPageThumbnails as bookmarks.  When first created they are not sensitive, but an item in their morph menu allows one to enable or disable the bookmark action.

Also included is a change so that the normal goToPage code, that chooses an inverse effect when going to an earlier page, is bypassed if triggered by a bookmark that specifies a transition effect.
"!

SketchMorph subclass: #BookPageThumbnailMorph
	instanceVariableNames: 'page pageNumber bookMorph flipOnClick '
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Morphic-Books'!

!BookMorph methodsFor: 'navigation' stamp: 'di 12/23/1998 16:20'!
goToPage: pageNumber transitionSpec: transitionSpec
	| pageIndex  aWorld oldPageNumber ascending newPage tSpec |
	pages isEmpty ifTrue: [^ self].

	pageIndex _ pageNumber asInteger.
	pageNumber < 1 ifTrue: [pageIndex _ pages size].
	pageNumber > pages size ifTrue: [pageIndex _ 1].
	newPage _ pages at: pageIndex.

	tSpec _ transitionSpec ifNil:  "If transition not specified by requestor..."
		[newPage valueOfProperty: #transitionSpec  " ... then consult new page"
			ifAbsent: [self transitionSpecFor: self  " ... otherwise this is the default"]].

	oldPageNumber _ pages indexOf: currentPage ifAbsent: [nil].
	ascending _ ((oldPageNumber == nil) or: [newPage == currentPage])
			ifTrue: [nil]
			ifFalse: [oldPageNumber < pageIndex].
	(aWorld _ self world) ifNotNil:
		[self primaryHand newKeyboardFocus: nil].

	self currentPage ~~ nil
		ifTrue:
		[(((pages at: pageIndex) owner isKindOf: TransitionMorph)
			and: [(pages at: pageIndex) isInWorld])
			ifTrue: [^ self  "In the process of a prior pageTurn"].
		self currentPlayerDo: [:aPlayer | aPlayer runAllClosingScripts].
		ascending ifNotNil:
			["Show appropriate page transition and start new page when done"
			currentPage stopStepping.
			(pages at: pageIndex) position: currentPage position.
			^ (TransitionMorph
					effect: tSpec second
					direction: tSpec third
					inverse: (ascending or: [transitionSpec notNil]) not)
				showTransitionFrom: currentPage
				to: (pages at: pageIndex)
				in: self
				whenStart: [self playPageFlipSound: tSpec first]
				whenDone:
					[currentPage delete; fullReleaseCachedState.
					self addMorphBack: (currentPage _ pages at: pageIndex).
					self snapToEdgeIfAppropriate.
					aWorld ifNotNil: [self world startSteppingSubmorphsOf: currentPage].
					self currentPlayerDo: [:aPlayer | aPlayer runAllOpeningScripts].
					(aWorld _ self world) ifNotNil: ["WHY??" aWorld displayWorld]]].

		"No transition, but at least decommission current page"
		currentPage delete; fullReleaseCachedState].

	self addMorphBack: (currentPage _ pages at: pageIndex).
	self snapToEdgeIfAppropriate.
	aWorld ifNotNil: [self world startSteppingSubmorphsOf: currentPage].
	self currentPlayerDo: [:aPlayer | aPlayer runAllOpeningScripts].
	(aWorld _ self world) ifNotNil: ["WHY??" aWorld displayWorld].
! !


!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 15:51'!
addCustomMenuItems: aCustomMenu hand: aHandMorph

	super addCustomMenuItems: aCustomMenu hand: aHandMorph.
	flipOnClick
		ifTrue: [aCustomMenu add: 'disable bookmark action' action: #toggleBookmark]
		ifFalse: [aCustomMenu add: 'enable bookmark action' action: #toggleBookmark].
	(bookMorph isKindOf: BookMorph)
		ifTrue:
			[aCustomMenu add: 'set page sound' action: #setPageSound:.
			aCustomMenu add: 'set page visual' action: #setPageVisual:]
! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 15:44'!
doPageFlip
	"Flip to this page"
	| tSpec |
	tSpec _ self valueOfProperty: #transitionSpec.
	tSpec ifNil: [bookMorph goToPage: pageNumber]
		ifNotNil: [bookMorph goToPage: pageNumber transitionSpec: tSpec]! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 15:57'!
encounteredAtTime: ticks inScorePlayer: scorePlayer atIndex: index inEventTrack: track secsPerTick: secsPerTick
	"Flip to this page with no extra sound"
	BookMorph turnOffSoundWhile: [self doPageFlip]! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 16:01'!
handlesMouseDown: event

	^ flipOnClick and: [event shiftPressed not and: [event controlKeyPressed not]]! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 15:45'!
initialize
	| f |
	super initialize.
	flipOnClick _ false.
	color _ Color lightGray.  "background color"
	f _ Form extent: 60@80 depth: Display depth.
	f fill: f boundingBox fillColor: color.
	self form: f.
! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 15:56'!
mouseDown: event

	self doPageFlip! !

!BookPageThumbnailMorph methodsFor: 'all' stamp: 'di 12/23/1998 15:53'!
toggleBookmark
	"Enable or disable sensitivity as a bookmark
		enabled means that a normal click will cause a pageFlip
		disabled means this morph can be picked up normally by the hand."

	flipOnClick _ flipOnClick not! !


